import csv


class PsychoPyCSV():
    """A simple abstraction for an in memory PsychoPy csv output file"""
    def __init__(self, filename):
        super().__init__()
        self.filename = filename
        self._load(filename)

    def _load(self, filename):
        """load the file contents into the events attribute"""
        with open(filename) as fp:
            reader = csv.DictReader(fp)
            self.events = list(reader)


def load(csvfile):
    """Load a psychopy csv into memory and expose the rows as events"""
    return PsychoPyCSV(csvfile)



# coding: utf-8
from __future__ import division
from prespy.scla.sndan import extract_sound_events, ExtractError, timing, stdStats
from .psychcsv import load


def scla(soundfile=None, logfile=None, **kwargs):
    """Implements similar logic to Neurobehavioural Systems SCLA program"""
    log = load(logfile)

    fs, pcodes, snds, port = extract_sound_events(soundfile, **kwargs)
    if (len(log.events) != len(pcodes)) or (len(pcodes) != len(snds)):
        raise ExtractError(log.events, pcodes, snds)

    datasets = {}
    for code, snd in zip(pcodes, snds):
        datasets['Port Snd Diffs'].append(snd - code)
    td, pl = timing(port, pcodes, snds, fs, **kwargs)
    datasets['Port Time Diffs'] = td['pcodes']
    datasets['Snd Time Diffs'] = td['snds']
    datasets['Port Code Lengths'] = pl
#    import pdb; pdb.set_trace()
    return stdStats(datasets)




__version__ = '0.0.3'

__title__ = 'psychosnd'

__summary__ = 'Psychopy sound analysis scripts'

__uri__ = 'https://github.com/gjcooper/psychosnd'

__license__ = 'GPLv3'

__author__ = 'Gavin Cooper'

__email__ = 'gjcooper@gmail.com'




from psychosnd.scla import scla, ExtractError
from psychosnd.__about__ import __version__, __title__
import sys
import argparse


def scla_script():
    parser = argparse.ArgumentParser(description='scla {} ({} variant) - Analyse sound latencies'.format(__version__, __title__))
    parser.add_argument('--runid', '-i', help='An id for the sound plot', default='scla')
    parser.add_argument('soundfile', help='A sound recording of port and sound output')
    parser.add_argument('logfile', help='Stimulus delivery recording of event sequence')
    parser.add_argument('--schannel', '-c', help='The channel sounds were recorded in', default=1)
    parser.add_argument('--maxdur', '-m', help='Max duration for sound/port event', default=0.012)
    parser.add_argument('--thresh', '-t', help='Threshold for sound/port detection', default=0.2)
    parser.add_argument('--version', action='version', version='scla {} ({} variant)'.format(__version__, __title__))

    args = parser.parse_args()

    try:
        res = scla(**vars(args))
    except ExtractError as e:
        lengths = map(len, [e.logData, e.portData, e.sndData])
        print('{}\nLogEvts: {} PortEvts: {} SndEvts: {}'.format(e.message, *lengths))
        sys.exit(65)
    report = ['==================================']
    for result in res:
        report.append(result)
        for measure in ['mean', 'min', 'max', 'stddev']:
            report.append('\t{}: {}'.format(measure, res[result][measure]))
        report.append('----------------------------------')
    report.append(report[0])

    print('\n'.join(report))
